
#include "worker.h"
#include "pyremoter.h"
#include <kr2/utils/configuration_file.h>

class Uploader : public Worker {

	// Do we need the Builder interface?..

	// Uploader& withCredFname(const std::string &a_cred_fname)
	// {
	// 	cred_fname = a_cred_fname;
	// 	return *this;
	// }

	// Uploader& withHost(const std::string &a_host)
	// {
	// 	host = a_host;
	// 	return *this;
	// }

	public:
		enum UploaderType {
			LOGS_UPLOADER,
			DIAGS_UPLOADER,
			CALIB_UPLOADER,

			SIZE
		};

		void add_type(const uint8_t &a_type)				{ types.push_back(a_type);  }

		void set_host(const std::string &a_host)            { host = a_host; }
		std::string get_host()                              { return this->host; }

		void set_config(const ConfifurationFile::PARSED_DATA &a_config)          { config = a_config; }
		ConfifurationFile::PARSED_DATA get_config()                              { return this->config; }

		void set_program_api(const boost::shared_ptr<kr2_program_api::ProgramInterface>& a_program_api) { program_api_ = a_program_api; }

		/// @brief Get the latest status of the job
		CBUN_PCALL get_status()                             { return this->status_; }

		/// @brief Reset the Uploader
		void reset() {
			this->status_ = kr2_program_api::CmdResult<>::Exception(7, PYREMOTER_STATUS_SDESC[7]);
			this->types.clear();
		}
    private:
		
		/// implement run for your specific Worker
		void run() {
			CBUN_PCALL running_status_ = this->status_;
			for (uint8_t type : types){
				if (type == UploaderType::LOGS_UPLOADER){
					running_status_ = baseSshExportLogsIniWithConfig(this->config, host);
				}
				if (type == UploaderType::DIAGS_UPLOADER){
					saveDashboardJsonToFile();
					running_status_ = baseSshExportDiagnosticsIni(this->config, host);
				}
				if (type == UploaderType::CALIB_UPLOADER){
					saveCalibrationToFile();
					running_status_ = baseSshExportCalibrationDataIni(this->config, host);
				}

				if (running_status_.result_ != running_status_.OK){
					break;
				}
			}
			this->status_ = running_status_;
			this->finished();
		}

		bool saveToFile(std::string a_path, std::string a_content)
		{
			std::ofstream datFile;
			
			datFile.open(a_path);
			
			if (datFile.good()) {
				datFile.write(a_content.c_str(), a_content.length());
				datFile.close();
			} else {
				return false;
			}
			
			return true;
		}

		bool saveDashboardJsonToFile(){
			usleep(kord_period);
			program_api_->rc_api_->spin();
			boost::shared_ptr<kr2rc_api::Controller::ConfigFileContent> read_config = program_api_->rc_api_->system_controller_->read_ConfigFile(kr2rc_api::Controller::ConfigFileRequest(kr2rc_api::Controller::ConfigFileRequest::EConfigFileId::eRC_DASHBOARD_JSON));
			CLOG_INFO("config read");
			if (read_config && read_config->valid()){
			    CLOG_INFO("valid data received");
			    std::string config_file(read_config->data().get(), read_config->dataLen()-1);
			    CLOG_INFO(config_file);
			    std::string ffolder = "/tmp/";
			    std::string fname = "dashboard.json";
			    saveToFile(ffolder + fname, config_file);
			    CLOG_INFO("after save");
			    return true;
			}
			CLOG_INFO("not valid or null");
			return false;
		}

		bool saveCalibrationToFile(){
			boost::shared_ptr<kr2rc_api::Controller::ConfigFileContent> read_config = program_api_->rc_api_->system_controller_->read_ConfigFile(kr2rc_api::Controller::ConfigFileRequest(kr2rc_api::Controller::ConfigFileRequest::EConfigFileId::eARM_MODEL_URDF));
			CLOG_INFO("file read");
			if (read_config && read_config->valid()){
			    CLOG_INFO("valid data received");
			    std::string config_file(read_config->data().get(), read_config->dataLen()-1);
			    CLOG_INFO(config_file);
			    std::string ffolder = "/tmp/";
			    std::string fname = "model.urdf";
			    saveToFile(ffolder + fname, config_file);
			    CLOG_INFO("after save");
			    return true;
			}
			CLOG_INFO("not valid or null");
			return false;
		}

        std::string host       = "";
		ConfifurationFile::PARSED_DATA config;
		CBUN_PCALL status_     = kr2_program_api::CmdResult<>::Exception(7, PYREMOTER_STATUS_SDESC[7]);

		boost::shared_ptr<kr2_program_api::ProgramInterface> program_api_;

		unsigned int kord_period = 4000; // micros



		std::vector<uint8_t> types;
};