/*////////////////////////////////////////////////////////////////////////////
//
// (C) Copyright 2022 by Kassow Robots, ApS
//
// The information contained herein is confidential, proprietary to Kassow Robots,
// ApS, and considered a trade secret as defined in section 263 and section 264
// under the Danish Criminal Code. Use of this information by anyone
// other than authorized employees of Kassow Robots, ApS is granted only under a
// written non-disclosure agreement, expressly prescribing the scope and
// manner of such use.
//
// Authors: Ondrej Bruna  obr@kassowrobots.com
//
/////////////////////////////////////////////////////////////////////////////*/

#include "kr2/kord/robot_io.h"

#include <algorithm>
#include <iostream>

namespace kr2 {

namespace kord {

using namespace kr2rc_api;

RobotIO::RobotIO() {
    reset();
}

void RobotIO::reset(){
    inputs_register_ = 0;
    analog_inputs_.fill(NAN);

    outputs_register_ = 0;
    analog_outputs_.fill(NAN);
}

void RobotIO::setOutputDigital(const std::vector<Digital>& a_digital_outputs)
{
    // std::cout << "Setting...." << std::endl;

    unsigned int offset_iob_digital     = num_outputs_relay_iob;
    unsigned int offset_iob_safe_output = num_outputs_relay_iob + num_outputs_digital_iob;
    unsigned int offset_tio_digital     = num_outputs_relay_iob + num_outputs_digital_iob + num_outputs_digital_safe_iob;
    unsigned int offset_tio_psu         = num_outputs_relay_iob + num_outputs_digital_iob + num_outputs_digital_safe_iob + num_outputs_digital_tio;

    DUID duid;
    for (int i = 0; i < num_outputs_relay_iob; i++) {
        duid = IOData::SysId::OPORT(i, IOData::SysId::IOBOARD, IOData::SysId::RELAY);
        auto di = std::find_if(a_digital_outputs.begin(), a_digital_outputs.end(), [duid](const Digital& adi){ return adi.gpio_id_ == duid;});
        if ( di != a_digital_outputs.end() ){
            // std::cout << "relay! duid:" << duid << std::endl;
            registerUpdate(outputs_register_, *di, i);
        }
    }

    for (int i = 0; i < num_outputs_digital_iob; i++) {
        duid = IOData::SysId::OPORT(i, IOData::SysId::IOBOARD, IOData::SysId::DIGITAL);
        auto di = std::find_if(a_digital_outputs.begin(), a_digital_outputs.end(), [duid](const Digital& adi){ return adi.gpio_id_ == duid;});
        if ( di != a_digital_outputs.end() ){
            // std::cout << "digital! duid:" << duid << std::endl;
            // assign i-th value to the IOIntRegister
            registerUpdate(outputs_register_, *di, i + num_outputs_relay_iob);
        }
    }

    // for (int i = 0; i < num_outputs_digital_safe_iob; i++) {
    //     duid = IOData::SysId::OPORT(i, IOData::SysId::IOBOARD, IOData::SysId::);
    //     auto di = std::find_if(a_digital_inputs.begin(), a_digital_inputs.end(), [duid](const Digital& adi){ return adi.gpio_id_ == duid;});
    //     if ( di != a_digital_inputs.end() ){
    //         // assign i-th value to the IOIntRegister
    //         registerUpdate(digital_outputs_, *di, i + num_outputs_relay_iob);
    //     }
    // }

    for (int i = 0; i < num_outputs_digital_tio; i++) {
        duid = IOData::SysId::OPORT(i, IOData::SysId::TOOLBOARD, IOData::SysId::DIGITAL);
        auto di = std::find_if(a_digital_outputs.begin(), a_digital_outputs.end(), [duid](const Digital& adi){ return adi.gpio_id_ == duid;});
        if ( di != a_digital_outputs.end() ){
            // std::cout << "tio! duid:" << duid << std::endl;
            // assign i-th value to the IOIntRegister
            registerUpdate(outputs_register_, *di, i + offset_tio_digital);
        }
    }

    for (int i = 0; i < num_outputs_psu_tio; i++) {
        duid = IOData::SysId::OPORT(i, IOData::SysId::TOOLBOARD, IOData::SysId::PSU);
        auto di = std::find_if(a_digital_outputs.begin(), a_digital_outputs.end(), [duid](const Digital& adi){ return adi.gpio_id_ == duid;});
        if ( di != a_digital_outputs.end() ){
            // std::cout << "psu tio! duid:" << duid << std::endl;
            // assign i-th value to the IOIntRegister
            registerUpdate(outputs_register_, *di, i + offset_tio_psu);
        }
    }
}

void RobotIO::setOutputAnalog(const std::vector<Analog>&) 
{

    // num_inputs_current_tio x1
    // num_inputs_voltage_tio x2
    // num_outputs_analog_tio x2
    // 
}

void RobotIO::setInputDigital(const std::vector<Digital>&a_digital_inputs)
{

    unsigned int offset_input_iob_digital = 0;
    unsigned int offset_input_iob_safe = num_inputs_digital_iob;
    
    DUID duid;
    for (int i = 0; i < num_inputs_digital_iob; i++){
        duid = IOData::SysId::IPORT(i, IOData::SysId::IOBOARD, IOData::SysId::DIGITAL);
        auto di = std::find_if(std::begin(a_digital_inputs), std::end(a_digital_inputs), [duid](const Digital& adi){ return adi.gpio_id_ == duid;});
        if ( di != a_digital_inputs.end() ){
            registerUpdate(inputs_register_, *di, i);
        }
    }
}

void RobotIO::setInputAnalog(const std::vector<Analog>&)
{

}

RobotIO::IORegister RobotIO::getOutputRegister() const
{
    return outputs_register_;
}

RobotIO::IORegister RobotIO::getInputRegister() const
{
    return inputs_register_;
}

std::array<double, num_analog_outputs> RobotIO::getOutputAnalog() const
{
    std::array<double, num_analog_outputs> returnValue;
    return returnValue;
}

std::array<double, num_analog_inputs> RobotIO::getInputAnalog() const
{
    std::array<double, num_analog_inputs> returnValue;
    return returnValue;
}

void RobotIO::registerUpdate(IORegister& a_reg, const Digital& a_kr2_gpio, unsigned int a_offset)
{
    if (a_kr2_gpio.value_) {
        registerSetBit(a_reg, a_offset);
    }
    else {
        registerResetBit(a_reg, a_offset);
    }
}

void RobotIO::registerSetBit(IORegister& a_register, unsigned int a_i)
{
    a_register |= (1 << a_i);
}

void RobotIO::registerResetBit(IORegister& a_register, unsigned int a_i)
{
    a_register &= ~(1 << a_i);
}

} // kord
} // kr2